// bessel1.h
// Bessel_1 Function
// Copyright (C) 1994,2001 Yukinori NAGATANI
// nagatani@yukawa.kyoto-u.ac.jp

#include <math.h>

#define	BESSEL_M	100
double rBessel1SmallZ (double Z)	// Able for Z < 35
{
	register int	m;
	double		Z2p4 = -Z*Z/4.0;
	double		Zm,	J;
	
	J  = 0.0;
	Zm = Z / 2.0;
	for (m = 1 ; m < BESSEL_M ; m++) {
		J  += Zm;
		Zm *= Z2p4 / ((double) m * ((double )m+1.0));
	}
	return	J;
}

double rBessel1LargeZ (double Z)	// Able for 15 < Z
{
	if (Z < 0.01)
		return 0.0;
	else
		return sqrt (2.0/M_PI/Z) * cos (Z - M_PI * 3/4);
}

double rBessel1 (double Z)
{
	double	a;
	if (Z < 15.0)
		return rBessel1SmallZ (Z);
	if (Z > 35.0)
		return rBessel1LargeZ (Z);
	a = (35.0 - Z) / (35.0 - 15.0);
	return	  rBessel1SmallZ (Z) * a 
		+ rBessel1LargeZ (Z) * (1.0 - a);
}

void CheckBessel1 (void)
{
	double	rZ;
	for (rZ = 0.0 ; rZ < 100.0 ; rZ += 0.5) 
		printf ("J1(%10.5lf) = %10.5lf  %10.5lf %10.5lf\n",
			rZ,
			rBessel1( rZ),
			rBessel1LargeZ( rZ),
		        rBessel1SmallZ( rZ));
}
