// kgprop.c
// Klein Goldon Reterted Propagation
// Audio Comvolution Integral
// Copyright (C) 1994,2001 Yukinori NAGATANI
// nagatani@yukawa.kyoto-u.ac.jp

#include "kgimp.cc"

Real rGetConvolutionValue (const RealWave & Gr,
			   const RealWave & In, int m)
{
	int	n = (m >= Gr.sizeRec - 1 ? Gr.sizeRec - 1 : m);
	Real	f = 0.0;
	for (; n >= 0 && m-n < In.sizeRec ; n--)
		f += Gr.prBuffer [n] * In.prBuffer [m-n];
	
	if (!(m%1000))
		fprintf (stderr, "%5.2lf ", (Real) m / Gr.Header.sample_fq);
	
	return	f;
}

Real MakeConvolution (const RealWave & Gr,
		      const RealWave & In,
		      RealWave & Out)
{
	for (size_t n = 0 ; n < Out.size ; n++)
		Out.prBuffer [n] = rGetConvolutionValue (Gr, In, n);
	Out.sizeRec = Out.size;
}

const char * szTitleMessage =
	"Klein-Goldon Sound Propagation  by Y.Nagatani";

const char * szHelpMessage =
	"Usage: kgimp [-m <400:mass (Hz)>]\n"
	"             [-x <10:distance (sec)>]\n"
	"             [-T <3:impulse time size (sec)>]\n"
	"             [-v <0.05:output volume>]\n"
	" read wav file (16bit 44100Hz mono/stereo) from stdin\n"
	" out put wave file to stdout.";

void error (const char * szMessage) {
	fprintf (stderr, "kgprop:%s\n%s\n", szMessage, szHelpMessage);
	exit (1);
}

void main (int nArgc, char *pszArgv[])
{
	KleinGoldonInpulse	Impulse;
	Impulse.size = 44100 * 3;
	
	Real	rVolume = 0.05;
	
	nArgc--;
	pszArgv++;
	for (; nArgc > 0 ; nArgc--, pszArgv++) {
		if (!strcmp (*pszArgv, "-m")) {
			nArgc--;
			pszArgv++;
			if (nArgc <= 0)
				error ("no args for -m");
			Impulse.rMass = atof (*pszArgv) * (2 * M_PI);
			continue;
		}
		if (!strcmp (*pszArgv, "-x")) {
			nArgc--;
			pszArgv++;
			if (nArgc <= 0)
				error ("no args for -x");
			Impulse.rX = atof (*pszArgv);
			continue;
		}
		if (!strcmp (*pszArgv, "-T")) {
			nArgc--;
			pszArgv++;
			if (nArgc <= 0)
				error ("no args for -T");
			Impulse.size = (int) (44100 * atof (*pszArgv));
			continue;
		}
		if (!strcmp (*pszArgv, "-v")) {
			nArgc--;
			pszArgv++;
			if (nArgc <= 0)
				error ("no args for -v");
			rVolume = atof (*pszArgv);
			continue;
		}
		error ("Invalid Argument");
	}
	fprintf (stderr, "%s\n", szTitleMessage);
	
	size_t		size = Impulse.size;
	RealWave	In (size),	Out (size);
	
	// Read Wav File
	In.read ();
	
	// Create Klein-Goldon Impulse Wave
	Impulse.create ();
	
	fprintf (stderr,
		 "Impulse Size: %5.2lf (sec)  "
		 "Imput Data Size: %5.2lf (sec) \n"
		 "Output Volume: %6.4lf\n",
		 (Real) Impulse.sizeRec / Impulse.Header.sample_fq,
		 (Real) In.sizeRec / In.Header.sample_fq,
		 rVolume);
	Impulse.showParameter ();
	
	// Make Convolution Wave
	MakeConvolution (Impulse, In, Out);
	fprintf (stderr, "\n");
	
	// Out Put Wav File
	Out.write (rVolume);
}
